#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'
# rubocop: disable Metrics/ModuleLength

# docs
module Puppet
  # rubocop: disable Metrics/BlockLength
  Type.newtype(:wls_jms_topic) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-EOD
    This resource allows you to manage a Tpoic in a JMS Module of a WebLogic domain.

    Here is an example on how you should use this:

        wls_jms_topic { 'jmsClusterModule:Topic1':
          ensure            => 'present',
          balancingpolicy   => 'Round-Robin',
          defaulttargeting  => '0',
          deliverymode      => 'No-Delivery',
          destination_keys  => ['JMSPriority', 'JmsMessageId'],
          distributed       => 1,
          expirationpolicy  => 'Discard',
          forwardingpolicy  => 'Replicated',
          jndiname          => 'jms/Topic1',
          redeliverydelay   => '2000',
          redeliverylimit   => '2',
          subdeployment     => 'jmsServers',
          timetodeliver     => '-1',
          timetolive        => '300000',
          consumptionpaused => '0',
          insertionpaused   => '0',
          productionpaused  => '0',
        }

    **WARNING** When you want to `absent` a topic. you must ensure that
    the `distributed` parameter is specified.

        wls_jms_topic { 'jmsClusterModule:ErrorTopic':
          ensure      => 'absent',
          distributed => 1,
        }

    Failure to specify the correct value for `distributed`, will result
    in an error on removal of the topic.

    EOD

    bean_path('/JMSSystemResources/%{jmsmodule}/JMSResource/%{jmsmodule}/%{object_type}')

    validate do
      return if purging
      return if self[:ensure] == :absent # No checking when we are absenting
      fail "#{path}: Defaulttargeting and Subdeployment are both set, only one can be active for JMS components" if defaulttargeting == :"1" && subdeployment
      fail "#{path}: Either defaulttargeting or a subdeployment needs to be selected for JMS components" if defaulttargeting == :"0" && !subdeployment
    end

    on_create do
      Puppet.info "create #{name} "
      actual_type = distributed_to_path.chop
      actual_path = bean_path.subst(to_hash)
      fail "#{path}: Parent resource #{actual_path} doesn't exists." unless wls_config.exists?(actual_path)
      wls_config.create(actual_type, actual_path, self[key_name])
      nil
    end

    on_destroy do |_|
      if exists_distributed?
        wls_config.destroy('UniformDistributedTopic', bean_path.subst(as_distributed), self[key_name])
      elsif exists_non_distributed?
        wls_config.destroy('Topic', bean_path.subst(as_non_distributed), self[key_name])
      else
        fail "#{path}: topic doesn't exist."
      end
      nil
    end

    #
    # Because bean_path uses the variable object_path, we need to have it available all the time
    # This code adds it to the hash based on the setting of distributed.
    #
    def to_hash
      hash = super
      hash[:object_type] = distributed_to_path
      hash
    end

    def as_distributed
      to_hash.dup.merge(:object_type => 'UniformDistributedTopics')
    end

    def as_non_distributed
      to_hash.dup.merge(:object_type => 'Topics')
    end

    def exists_distributed?
      wls_config.exists?("#{bean_path.subst(as_distributed)}/#{self[key_name]}")
    end

    def exists_non_distributed?
      wls_config.exists?("#{bean_path.subst(as_non_distributed)}/#{self[key_name]}")
    end

    #
    # Only use elements on the path where this condition is true. The condition is
    # that is has specified serverTemplates
    #
    index_condition do |_name, path, _wls_config|
      %w[Topics UniformDistributedTopics].include?(path.split('/').last)
    end

    ensurable
    parameter :topic_name
    parameter :domain
    parameter :name
    parameter :jmsmodule
    parameter :timeout

    property :distributed
    property :jndiname
    property :subdeployment
    property :balancingpolicy
    property :quota
    property :defaulttargeting
    parameter :errordestinationtype
    property :errordestination
    property :expirationloggingpolicy
    property :redeliverylimit
    property :expirationpolicy
    property :redeliverydelay
    property :timetodeliver
    property :timetolive
    property :deliverymode
    property :messagelogging
    property :templatename
    property :forwardingpolicy
    property :destination_keys
    property :insertionpaused
    property :consumptionpaused
    property :productionpaused
    property :messages_limit_override
    extraproperties do
      property :multicast_port
      property :multicast_template_bean
      property :multicast_time_to_live
      property :override_delivery_mode
      property :override_priority
      property :override_redelivery_delay
      property :override_template_bean
      property :override_time_to_deliver
      property :override_time_to_live
      property :thresholds_bytes_high
      property :thresholds_bytes_low
      property :thresholds_messages_high
      property :thresholds_messages_low
      property :thresholds_template_bean
      property :attach_sender
      property :delivery_error_destination
      property :delivery_expiration_logging_policy
      property :delivery_expiration_policy
      property :delivery_redelivery_limit
      property :delivery_template_bean
      property :incomplete_work_expiration_time
      property :insertion_paused_at_startup
      property :jms_create_destination_identifier
      property :jndi_name
      property :local_jndi_name
      property :log_message_logging_enabled
      property :log_message_logging_format
      property :log_template_bean
      property :maximum_message_size
      property :production_paused_at_startup
      property :saf_export_policy
      property :consumption_paused_at_startup
      property :notes
      property :sub_deployment_name
      property :unit_of_order_routing
      property :default_targeting_enabled
      property :default_unit_of_order
      property :forwarding_policy
      property :load_balancing_policy
      property :messaging_performance_preference
      property :template
      property :unit_of_work_handling_policy
      #
      # Weblogic 12.2.1 new properties
      #
      property :multicast_address
    end

    add_title_attributes(:jmsmodule, :topic_name) do
      /(.+):(.+)/
    end

    def distributed_to_path
      case distributed
      when 0, '0' then 'Topics'
      when 1, '1' then 'UniformDistributedTopics'
      else
        fail 'Invalid value for distributed'
      end
    end
  end
end
